<?php
require_once 'config.php';

$latestData = [];
$recordValue = 0;
$targetValue = 0;
$firstTimestamp = '';
$lastTimestamp = '';
$totalEnergy = 0;
$dataExists = false;
$offset = 0;

// Aktuellen Offset-Wert aus der settings-Tabelle abrufen
$settingsSql = "SELECT * FROM settings WHERE id = 0 LIMIT 1";
$settingsResult = $conn->query($settingsSql);

if ($settingsResult->num_rows > 0) {
    $settingsRow = $settingsResult->fetch_assoc();
    $recordValue = $settingsRow['record'];
    $targetValue = $settingsRow['target'];
    $offset = $settingsRow['offset'];
    $lastConnection = $settingsRow['last_connection'];

}

// Letzten Messwert aus der Tabelle 'data' abrufen
$sqlLatest = "SELECT timestamp, power, voltage, current, energy, frequency FROM data ORDER BY timestamp DESC LIMIT 1";
$resultLatest = $conn->query($sqlLatest);

if ($resultLatest->num_rows > 0) {
    $latestData = $resultLatest->fetch_assoc();
    $totalEnergy = $latestData['energy'];
    $dataExists = true;
}

// Alle Messwerte aus der Tabelle 'data' abrufen
$sqlAll = "SELECT timestamp, power, voltage, current, energy, frequency FROM data ORDER BY timestamp ASC";
$resultAll = $conn->query($sqlAll);
$allData = [];

if ($resultAll->num_rows > 0) {
    $firstTimestamp = '';
    $lastTimestamp = '';
    while ($row = $resultAll->fetch_assoc()) {
        $allData[] = $row;
        if (empty($firstTimestamp)) {
            $firstTimestamp = $row['timestamp'];
        }
        $lastTimestamp = $row['timestamp'];
    }
}

if ($_SERVER["REQUEST_METHOD"] == "POST") {
    if (isset($_POST['toggle_record'])) {
        $newRecordValue = $recordValue == 1 ? 0 : 1;
        $updateRecordSql = "UPDATE settings SET record = $newRecordValue WHERE id = 0";
        $conn->query($updateRecordSql);

        if ($newRecordValue == 1) {
            $conn->query("TRUNCATE TABLE data");
        }

        header("Location: " . $_SERVER['PHP_SELF']);
        exit;
    }

    if (isset($_POST['set_target'])) {
        $newTargetValue = floatval($_POST['target_value']);
        $updateTargetSql = "UPDATE settings SET target = $newTargetValue WHERE id = 0";
        $conn->query($updateTargetSql);
        header("Location: " . $_SERVER['PHP_SELF']);
        exit;
    }
}

// Verbindung schlieen
$conn->close();

// Berechnung des Gesamten Energieverbrauchs (Wh) als energy - offset
$actualTotalEnergy = $totalEnergy - $offset;

$energyDifference = $actualTotalEnergy - $targetValue;
$percentageDifference = ($targetValue > 0) ? ($energyDifference / $targetValue) * 100 : 0;
$runtime = '';

if ($firstTimestamp && $lastTimestamp) {
    $datetime1 = new DateTime($firstTimestamp);
    $datetime2 = new DateTime($lastTimestamp);
    $interval = $datetime1->diff($datetime2);
    $runtime = $interval->format('%d Tage %H:%I:%S');
}
?>

<!DOCTYPE html>
<html lang="de">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Messwerte</title>
    <script src="https://cdn.jsdelivr.net/npm/chart.js"></script>
    <style>
        body { font-family: Arial, sans-serif; margin: 0; padding: 20px; }
        .container { display: flex; flex-direction: column; align-items: center; }
        .content { display: flex; width: 100%; justify-content: space-between; flex-wrap: wrap; }
        .content > div { flex: 1; padding: 10px; box-sizing: border-box; }
        .control-panel { width: 100%; padding: 10px; }
        table { border-collapse: collapse; width: 100%; margin-bottom: 20px; }
        
        th, td { border: 1px solid #ddd; padding: 8px; text-align: left; }
        th { background-color: #f2f2f2; }
        .button { display: block; width: 100%; padding: 10px; margin: 10px 0; text-align: center; border: none; cursor: pointer; }
        .button.start { background-color: green; color: white; }
        .button.stop { background-color: red; color: white; }
        .input-group { display: flex; align-items: center; gap: 5px; }
        .chart-container { width: 100%; margin-top: 20px; }
        @media (max-width: 768px)
</style>
</head>
<body>
    <div class="container">
        <div class="content">
            <div class="table-container">
                <?php if (!empty($latestData)) : ?>
                    <table>
                        <tr><th>Letzter Datensatz</th><th>Wert</th></tr>
                        <tr><td>Zeitstempel</td><td><?php echo $latestData['timestamp']; ?></td></tr>
                        <tr><td>Leistung</td><td><?php echo $latestData['power']; ?> W</td></tr>
                        <tr><td>Spannung</td><td><?php echo $latestData['voltage']; ?> V</td></tr>
                        <tr><td>Strom</td><td><?php echo $latestData['current']; ?> mA</td></tr>
                        <tr><td>Z&auml;hlerstand Total </td><td><?php echo $latestData['energy']; ?> Wh</td></tr>
                        <tr><td>Frequenz</td><td><?php echo $latestData['frequency']; ?> Hz</td></tr>
                    </table>
                <?php else : ?>
                    <p>Keine Messwerte in der Tabelle gefunden.</p>
                <?php endif; ?>
            </div>

            <div class="control-panel">
                <form method="post">
                    <button type="submit" name="toggle_record" class="button <?php echo $recordValue == 1 ? 'stop' : 'start'; ?>">
                        <?php echo $recordValue == 1 ? 'Aufnahme beenden' : 'Aufnahme starten'; ?>
                    </button>
                </form>

                <form method="post" class="input-group">
                    <label for="target_value" style="width: 180px;">Zielwert (Wh):</label>
                    <input type="number" name="target_value" id="target_value" max="999999" step="1" value="<?php echo $targetValue; ?>" style="width: 80px; padding: 5px;">
                    <button type="submit" name="set_target" class="button" style="background-color: blue; color: white;">Speichern</button>
                </form>

                <!-- Neuer Button zum Aktualisieren -->
                <form method="post">
                    <button type="submit" class="button" style="background-color: orange; color: white;">Daten aktualisieren</button>
                </form>
                
                   <!-- Anzeige des letzten Verbindungszeitpunkts -->
                <p id="last-connection">Letzte Daten&uuml;bertragung: <span id="connection-time"><?php echo $lastConnection; ?></span></p>
           
                
            </div>

            <?php if ($dataExists) : ?>
                <div class="table-container">
                    <table>
                        <tr><th>Messergebnisse</th><th>Wert</th></tr>
                        <tr><td><?php echo $recordValue == 1 ? 'Gestartet' : 'Gestartet / Beendet'; ?></td><td><?php echo $recordValue == 1 ? $firstTimestamp : "$firstTimestamp / $lastTimestamp"; ?></td></tr>
                        <tr><td>Laufzeit</td><td><?php echo $runtime; ?></td></tr>
                        <tr><td>Energieverbrauch</td><td><?php echo $actualTotalEnergy; ?> Wh</td></tr>
                        <?php if ($targetValue > 0) : ?>
                            <tr><td>Abweichung Zielwert</td><td><?php echo $energyDifference . " Wh (" . round($percentageDifference, 2) . " % )"; ?></td></tr>
                        <?php endif; ?>
                    </table>
                </div>
            <?php endif; ?>
        </div>

        <?php if ($dataExists) : ?>
            <div class="chart-container">
                <canvas id="dataChart"></canvas>
            </div>
        <?php endif; ?>
    </div>

    <?php if ($dataExists) : ?>
        <script>
            const labels = <?php echo json_encode(array_column($allData, 'timestamp')); ?>;
            const powerData = <?php echo json_encode(array_column($allData, 'power')); ?>;
            const voltageData = <?php echo json_encode(array_column($allData, 'voltage')); ?>;
            const currentData = <?php echo json_encode(array_column($allData, 'current')); ?>;
            const energyData = <?php echo json_encode(array_column($allData, 'energy')); ?>;
            const frequencyData = <?php echo json_encode(array_column($allData, 'frequency')); ?>;

            const ctx = document.getElementById('dataChart').getContext('2d');
            const dataChart = new Chart(ctx, {
                type: 'line',
                data: {
                    labels: labels,
                    datasets: [
                        { label: 'Leistung (W)', data: powerData, borderColor: 'rgba(255, 99, 132, 1)', fill: false, hidden: false },
                        { label: 'Verbrauch (Wh)', data: energyData, borderColor: 'rgba(54, 162, 235, 1)', fill: false, hidden: true },
                        { label: 'Spannung (V)', data: voltageData, borderColor: 'rgba(255, 206, 86, 1)', fill: false, hidden: true },
                        { label: 'Strom (mA)', data: currentData, borderColor: 'rgba(75, 192, 192, 1)', fill: false, hidden: true },
                        { label: 'Frequenz (Hz)', data: frequencyData, borderColor: 'rgba(153, 102, 255, 1)', fill: false, hidden: true }
                    ]
                },
                options: {
                    responsive: true,
                    scales: {
                        x: { display: true, title: { display: true, text: 'Zeitstempel' } },
                        y: { display: true, title: { display: true, text: 'Werte' } }
                    }
                }
            });
        </script>
    <?php endif; ?>
</body>
<script>
        function formatTimestamp(timestamp) {
            const date = new Date(timestamp);
            const now = new Date();
            let diff = Math.floor((now - date) / 1000);

            setInterval(() => {
                diff++;
                let timeText = '';
                if (diff < 120) {
                    timeText = `vor ${diff} Sekunden`;
                } else if (diff < 3600) {
                    timeText = `vor ${Math.floor(diff / 60)} Minuten`;
                } else if (diff < 86400) {
                    timeText = `vor ${Math.floor(diff / 3600)} Stunden`;
                } else {
                    timeText = date.toLocaleString();
                }
                document.getElementById('connection-time').innerText = timeText;
            }, 1000);
        }

        const lastConnectionElement = document.getElementById('connection-time').innerText;
        formatTimestamp(lastConnectionElement);
    </script>
</html>
